@@ -57,14 +57,6 @@ expose_stan_functions <- function(stanmodel) {
   lines <- gsub("vector<Eigen::Matrix<.*,Eigen::Dynamic> >", "vector<matrix_d>", lines)
   lines <- gsub("Eigen::Matrix<.*,Eigen::Dynamic>", "matrix_d", lines)
   
-  # restore Stan's Eigen typedefs that were clobbered by the previous lines
-  lines <- gsub("typedef vector_d vector_d;", 
-                "typedef Eigen::Matrix<double,Eigen::Dynamic,1> vector_d;", lines)
-  lines <- gsub("typedef row_vector_d row_vector_d;", 
-                "typedef Eigen::Matrix<double,1,Eigen::Dynamic> row_vector_d;", lines)
-  lines <- gsub("typedef matrix_d matrix_d;",
-                "typedef Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic> matrix_d;", lines)
-
   # kill foo_log<false> functions because of templating
   templated <- grep("_log<false>", lines, fixed = TRUE)
   if(length(templated) > 0) for(i in rev(templated)) {
@@ -156,13 +148,21 @@ expose_stan_functions <- function(stanmodel) {
   lines <- gsub("const static bool propto__ = true;",
                 "const static bool propto__ = false;", lines, fixed = TRUE)
   
-
+  # restore Stan's Eigen typedefs that were clobbered by the previous lines
+  lines <- sub("typedef vector_d vector_d;", "using stan::math::vector_d;", lines)
+  lines <- sub("typedef row_vector_d row_vector_d;", "using stan::math::row_vector_d;", lines)
+  lines <- sub("typedef matrix_d matrix_d;", "using stan::math::matrix_d;", lines)
+  
   # add dependencies
-  lines <- c("// [[Rcpp::depends(StanHeaders)]]", 
+  extras <- dir(rstan_options("boost_lib2"), pattern = "hpp$", 
+                full.names = TRUE, recursive = TRUE)
+  lines <- c("// [[Rcpp::depends(StanHeaders)]]",
              "// [[Rcpp::depends(BH)]]",
              "// [[Rcpp::depends(RcppEigen)]]",
              "#include<Rcpp.h>",
              "#include<RcppEigen.h>",
+             if (length(extras) > 0) sapply(extras, FUN = function(x)
+               paste0("#include<", x, ">")),             
              "#include<stan/math.hpp>",
              "#include <src/stan/lang/rethrow_located.hpp>",
 
